/*---------------------------------------------------------------------------
 * 2005 NORSYS
 * main author : nono
 *
 * This software is a computer program whose purpose is to provide abstraction
 * for accessing directory data sources within java applications.
 *
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 * Created on Oct 19, 2005
 * --------------------------------------------------------------------------*/
package speculoos.jndi;

import java.util.Map;

import speculoos.core.Mapper;
import speculoos.core.MapperException;


/**
 * A mapper that allows comparison of attributes of a single node with
 * what is stored in the source.
 * <p>
 * This mapper implements the compare operation on JNDI/LDAP sources which
 * allows comparison of values stored in the data source without 
 * actual retrieval of the value. This operation is mainly used for 
 * authentication purposes.
 * All the  attributes in the input map are  compared with their value in the source,
 * the output map being used to map names in object space
 * to names in directory space.
 * </p>
 * @author nono
 * @version $Id: CompareMapper.java 259 2006-05-23 10:34:50Z /C=FR/ST=Nord/L=Lille/O=Norsys SA/OU=UE/CN=Arnaud Bailly/emailAddress=abailly@norsys.fr $
 */
public interface CompareMapper extends Mapper {

    /**
     * Execute a compare query on the underlying LDAP data source.
     * This kind of query compares the attributes defined in the <code>input</code> map
     * and the attributes stored in the source for one node. If they all
     * match (ie. are equals) then this method returns true, if at least one
     * matching fails, this method returns false;
     * 
     * @param input an object to compare value to.
     * @param env local environment for query customization.
     * @return true if comparison succeeds for each attribute in input map.
     * @throws MapperException if an input attribute fails validation rules checking
     * or a technical exception occurs.
     */
    boolean compare(Object input, Map env) throws MapperException;
    
}
