/*---------------------------------------------------------------------------
 * 2005 NORSYS
 * main author : nono
 *
 * This software is a computer program whose purpose is to provide abstraction
 * for accessing directory data sources within java applications.
 *
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 * Created on Oct 19, 2005
 * --------------------------------------------------------------------------*/
package speculoos.jndi;

import java.util.List;
import java.util.Map;

import speculoos.core.Mapper;
import speculoos.core.MapperException;


/**
 * This interface allows search operations on JNDI source.
 * <p>
 * A search operation allows retrieval of one or more nodes from
 * a JNDI/LDAP data source according to some criteria. The method 
 *  {@link #search(Map, Map)} constructs a list of output Map objects 
 *  representing attributes of nodes in a data source which match
 *  some filter as given by input map parameter.
 * </p>
 * <p>
 * The <code>env</code> parameter may be used to further control 
 * the query (eg. Limiting number of returned objects, timeout,...)
 * </p>
 * 
 * @author nono
 * @version $Id: SearchMapper.java 259 2006-05-23 10:34:50Z /C=FR/ST=Nord/L=Lille/O=Norsys SA/OU=UE/CN=Arnaud Bailly/emailAddress=abailly@norsys.fr $
 * @see <a href="/doc/conception/spec-api.html">API documentation</a>
 */
public interface SearchMapper extends Mapper {

    /**
     * Uses input map as filter parameters for searching a data source
     * and outputs a list of matching nodes.
     *  
     * @param input an object describing values to retrieve.
     * @param env the local environment to use. May be null.
     * @return a List<Map> object containing result of query. Elements 
     * of the list conform to the output map parameter as configured
     * in initialization of the source. List may be empty.
     * @throws MapperException if input map does not contains required 
     * parameters for the filter or if a validation rule is broken.
     */
    List search(Object input, Map env) throws MapperException;

    /**
     * Redeclaration of Object's clone.
     */
    Object clone();
}
