/*---------------------------------------------------------------------------
 * �2005 NORSYS
 * main author : nono
 *
 * This software is a computer program whose purpose is to provide abstraction
 * for accessing directory data sources within java applications.
 *
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 * Created on Nov 2, 2005
 * --------------------------------------------------------------------------*/

package speculoos.jndi.mappers;

import java.util.Map;

import javax.naming.NamingException;
import javax.naming.directory.SearchControls;

import speculoos.core.MapperException;
import speculoos.jndi.CompareMapper;

/**
 * Implementation of the compare LDAP operation.
 * <p>
 * A compare LDAP query is like a search query on a single object that does not
 * return attributes. For example, it allows comparison of password without
 * actual retrieval of the passwod.
 * </p>
 * 
 * @author nono
 * @version $Id$
 */
public class CompareMapperImpl extends JNDIMapper implements CompareMapper {

	private FilterFromInput filter;

	/**
	 * @param value
	 */
	public CompareMapperImpl(String value) {
		super(value);
	}

	/**
	 * 
	 */
	public CompareMapperImpl() {
		super("compare");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.norsys.mapper.jndi.CompareMapper#compare(java.util.Map,
	 *      java.util.Map)
	 */
	public boolean compare(Object input, Map env) throws MapperException {
		return ((Boolean) map(input, env)).booleanValue();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see speculoos.jndi.mappers.JNDIMapper#doMap(java.lang.Object,
	 *      java.util.Map)
	 */
	public Object doMap(Object input, Map env) throws MapperException {
		/* instanciate fitler */
		String _filter = (String) env.get("filter");
		if (_filter == null && filter != null) {
			filter.map(input, env);
			_filter = (String) env.get("filter");
		}
		if (_filter == null)
			throw new MapperException(getName()
					+ ": No filter definable for this search");
		/* control */
		SearchControls controls = new SearchControls();
		controls.setSearchScope(SearchControls.OBJECT_SCOPE);
		controls.setReturningAttributes(new String[0]);
		controls.setReturningObjFlag(true);
		/* run query */
		try {
			if (log.isDebugEnabled())
				log.debug("[" + getName() + "] comparing " + _root + " with "
						+ _filter);
			directory.search(_root, _filter, controls);
			if (log.isDebugEnabled())
				log.debug("[" + getName() + "] done");
		} catch (NamingException ne) {
			throw new MapperException("Failed to match requested attributes",
					ne);
		}
		return Boolean.TRUE;
	}

	public void setFilter(Class cls, Map map) {
		this.filter = FilterFromInput.create(cls, map);
	}

}
